package com.jjt.calc.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.enums.BusinessType;
import com.jjt.calc.domain.TwinCalcHourSpec;
import com.jjt.calc.service.ITwinCalcHourSpecService;
import com.jjt.common.utils.poi.ExcelUtil;
import com.jjt.common.core.page.TableDataInfo;

/**
 * 按配方1小时统计数据Controller
 *
 * @author wukai
 * @date 2025-01-18
 */
@Api(tags="按配方1小时统计数据")
@RestController
@RequestMapping("/calc/calcSpec")
public class TwinCalcHourSpecController extends BaseController{
    @Resource
    private ITwinCalcHourSpecService twinCalcHourSpecService;

    /**
     * 查询按配方1小时统计数据列表
     */
    @ApiOperation("查询按配方1小时统计数据列表")
    @PreAuthorize("@ss.hasPermi('calc:calcSpec:list')")
    @GetMapping("/list")
    public TableDataInfo list(TwinCalcHourSpec twinCalcHourSpec)
    {
        startPage();
        List<TwinCalcHourSpec> list = twinCalcHourSpecService.selectTwinCalcHourSpecList(twinCalcHourSpec);
        return getDataTable(list);
    }

    /**
     * 导出按配方1小时统计数据列表
     */
    @ApiOperation("导出按配方1小时统计数据列表")
    @PreAuthorize("@ss.hasPermi('calc:calcSpec:export')")
    @Log(title = "按配方1小时统计数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TwinCalcHourSpec twinCalcHourSpec)
    {
        List<TwinCalcHourSpec> list = twinCalcHourSpecService.selectTwinCalcHourSpecList(twinCalcHourSpec);
        ExcelUtil<TwinCalcHourSpec> util = new ExcelUtil<TwinCalcHourSpec>(TwinCalcHourSpec.class);
        util.exportExcel(response, list, "按配方1小时统计数据数据");
    }

    /**
     * 获取按配方1小时统计数据详细信息
     */
    @ApiOperation("获取按配方1小时统计数据详细信息")
    @PreAuthorize("@ss.hasPermi('calc:calcSpec:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(twinCalcHourSpecService.selectTwinCalcHourSpecById(id));
    }

    /**
     * 新增按配方1小时统计数据
     */
    @ApiOperation("新增按配方1小时统计数据")
    @PreAuthorize("@ss.hasPermi('calc:calcSpec:add')")
    @Log(title = "按配方1小时统计数据", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TwinCalcHourSpec twinCalcHourSpec)
    {
        return toAjax(twinCalcHourSpecService.insertTwinCalcHourSpec(twinCalcHourSpec));
    }

    /**
     * 修改按配方1小时统计数据
     */
    @ApiOperation("修改按配方1小时统计数据")
    @PreAuthorize("@ss.hasPermi('calc:calcSpec:edit')")
    @Log(title = "按配方1小时统计数据", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TwinCalcHourSpec twinCalcHourSpec)
    {
        return toAjax(twinCalcHourSpecService.updateTwinCalcHourSpec(twinCalcHourSpec));
    }

    /**
     * 删除按配方1小时统计数据
     */
    @ApiOperation("删除按配方1小时统计数据")
    @PreAuthorize("@ss.hasPermi('calc:calcSpec:remove')")
    @Log(title = "按配方1小时统计数据", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(twinCalcHourSpecService.deleteTwinCalcHourSpecByIds(ids));
    }
}
