package com.jjt.calc.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.enums.BusinessType;
import com.jjt.calc.domain.TwinCalcStop;
import com.jjt.calc.service.ITwinCalcStopService;
import com.jjt.common.utils.poi.ExcelUtil;
import com.jjt.common.core.page.TableDataInfo;

/**
 * 停机数据统计Controller
 *
 * @author wukai
 * @date 2025-01-18
 */
@Api(tags="停机数据统计")
@RestController
@RequestMapping("/calc/calcStop")
public class TwinCalcStopController extends BaseController{
    @Resource
    private ITwinCalcStopService twinCalcStopService;

    /**
     * 查询停机数据统计列表
     */
    @ApiOperation("查询停机数据统计列表")
    @PreAuthorize("@ss.hasPermi('calc:calcStop:list')")
    @GetMapping("/list")
    public TableDataInfo list(TwinCalcStop twinCalcStop)
    {
        startPage();
        List<TwinCalcStop> list = twinCalcStopService.selectTwinCalcStopList(twinCalcStop);
        return getDataTable(list);
    }

    /**
     * 导出停机数据统计列表
     */
    @ApiOperation("导出停机数据统计列表")
    @PreAuthorize("@ss.hasPermi('calc:calcStop:export')")
    @Log(title = "停机数据统计", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TwinCalcStop twinCalcStop)
    {
        List<TwinCalcStop> list = twinCalcStopService.selectTwinCalcStopList(twinCalcStop);
        ExcelUtil<TwinCalcStop> util = new ExcelUtil<TwinCalcStop>(TwinCalcStop.class);
        util.exportExcel(response, list, "停机数据统计数据");
    }

    /**
     * 获取停机数据统计详细信息
     */
    @ApiOperation("获取停机数据统计详细信息")
    @PreAuthorize("@ss.hasPermi('calc:calcStop:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(twinCalcStopService.selectTwinCalcStopById(id));
    }

    /**
     * 新增停机数据统计
     */
    @ApiOperation("新增停机数据统计")
    @PreAuthorize("@ss.hasPermi('calc:calcStop:add')")
    @Log(title = "停机数据统计", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TwinCalcStop twinCalcStop)
    {
        return toAjax(twinCalcStopService.insertTwinCalcStop(twinCalcStop));
    }

    /**
     * 修改停机数据统计
     */
    @ApiOperation("修改停机数据统计")
    @PreAuthorize("@ss.hasPermi('calc:calcStop:edit')")
    @Log(title = "停机数据统计", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TwinCalcStop twinCalcStop)
    {
        return toAjax(twinCalcStopService.updateTwinCalcStop(twinCalcStop));
    }

    /**
     * 删除停机数据统计
     */
    @ApiOperation("删除停机数据统计")
    @PreAuthorize("@ss.hasPermi('calc:calcStop:remove')")
    @Log(title = "停机数据统计", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(twinCalcStopService.deleteTwinCalcStopByIds(ids));
    }
}
