package com.jjt.calc.service;

import com.jjt.calc.domain.TwinCalcHour;

import java.util.Date;
import java.util.List;

/**
 * 1小时统计数据Service接口
 *
 * @author wukai
 * @date 2025-01-18
 */
public interface ITwinCalcHourService {
    /**
     * 查询1小时统计数据
     *
     * @param id 1小时统计数据主键
     * @return 1小时统计数据
     */
    public TwinCalcHour selectTwinCalcHourById(Long id);

    /**
     * 查询1小时统计数据列表
     *
     * @param twinCalcHour 1小时统计数据
     * @return 1小时统计数据集合
     */
    public List<TwinCalcHour> selectTwinCalcHourList(TwinCalcHour twinCalcHour);

    /**
     * 新增1小时统计数据
     *
     * @param twinCalcHour 1小时统计数据
     * @return 结果
     */
    public int insertTwinCalcHour(TwinCalcHour twinCalcHour);

    /**
     * 修改1小时统计数据
     *
     * @param twinCalcHour 1小时统计数据
     * @return 结果
     */
    public int updateTwinCalcHour(TwinCalcHour twinCalcHour);

    /**
     * 批量删除1小时统计数据
     *
     * @param ids 需要删除的1小时统计数据主键集合
     * @return 结果
     */
    public int deleteTwinCalcHourByIds(Long[] ids);

    /**
     * 删除1小时统计数据信息
     *
     * @param id 1小时统计数据主键
     * @return 结果
     */
    public int deleteTwinCalcHourById(Long id);

    /**
     * 统计上一时段数据
     */
    public void last();

    /**
     * 从数据库最后一个时间段统计至上一个时段
     */
    public void calc2Curr();

    /**
     * 统计指定日期指定时段数据
     *
     * @param date   指定日期 yyyy-mm-dd
     * @param period 时段
     */
    public void calc(String date, int period);

    /**
     * 按开始和结束时间查询数据
     *
     * @param sTime 开始时间
     * @param eTime 结束时间
     * @return 查询列表
     */
    List<TwinCalcHour> selectTwinCalcHourListByDate(Date sTime, Date eTime);

    /**
     * 获取数据库最后一条记录
     *
     * @return 对象
     */
    TwinCalcHour lastHour();
}
