package com.jjt.calc.service.impl;

import com.jjt.biz.domain.TwinDevice;
import com.jjt.calc.domain.TwinCalcAlarms;
import com.jjt.calc.domain.TwinRecordAlarms;
import com.jjt.calc.mapper.TwinCalcAlarmsMapper;
import com.jjt.calc.service.ITwinCalcAlarmsService;
import com.jjt.calc.service.ITwinRecordAlarmsService;
import com.jjt.common.utils.StringUtils;
import com.jjt.common.utils.bean.BeanUtils;
import com.jjt.system.service.ISysConfigService;
import org.apache.ibatis.session.ExecutorType;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 告警数据统计Service业务层处理
 *
 * @author wukai
 * @date 2025-01-18
 */
@Service
public class TwinCalcAlarmsServiceImpl implements ITwinCalcAlarmsService {
    @Resource
    private TwinCalcAlarmsMapper twinCalcAlarmsMapper;
    @Resource
    private ISysConfigService configService;
    @Resource
    private ITwinRecordAlarmsService alarmsService;
    @Resource
    private SqlSessionFactory factory;

    /**
     * 查询告警数据统计
     *
     * @param id 告警数据统计主键
     * @return 告警数据统计
     */
    @Override
    public TwinCalcAlarms selectTwinCalcAlarmsById(Long id) {
        return twinCalcAlarmsMapper.selectTwinCalcAlarmsById(id);
    }

    /**
     * 查询告警数据统计列表
     *
     * @param twinCalcAlarms 告警数据统计
     * @return 告警数据统计
     */
    @Override
    public List<TwinCalcAlarms> selectTwinCalcAlarmsList(TwinCalcAlarms twinCalcAlarms) {
        return twinCalcAlarmsMapper.selectTwinCalcAlarmsList(twinCalcAlarms);
    }

    /**
     * 新增告警数据统计
     *
     * @param twinCalcAlarms 告警数据统计
     * @return 结果
     */
    @Override
    public int insertTwinCalcAlarms(TwinCalcAlarms twinCalcAlarms) {
        return twinCalcAlarmsMapper.insertTwinCalcAlarms(twinCalcAlarms);
    }

    /**
     * 修改告警数据统计
     *
     * @param twinCalcAlarms 告警数据统计
     * @return 结果
     */
    @Override
    public int updateTwinCalcAlarms(TwinCalcAlarms twinCalcAlarms) {
        return twinCalcAlarmsMapper.updateTwinCalcAlarms(twinCalcAlarms);
    }

    /**
     * 批量删除告警数据统计
     *
     * @param ids 需要删除的告警数据统计主键
     * @return 结果
     */
    @Override
    public int deleteTwinCalcAlarmsByIds(Long[] ids) {
        return twinCalcAlarmsMapper.deleteTwinCalcAlarmsByIds(ids);
    }

    /**
     * 删除告警数据统计信息
     *
     * @param id 告警数据统计主键
     * @return 结果
     */
    @Override
    public int deleteTwinCalcAlarmsById(Long id) {
        return twinCalcAlarmsMapper.deleteTwinCalcAlarmsById(id);
    }

    /**
     * 告警数据治理，从告警记录表到告警统计表中
     *
     * @param startTime  开始时间
     * @param deviceList 设备列表
     */
    @Override
    public void process(LocalDateTime startTime, List<TwinDevice> deviceList) {
        String config = configService.selectConfigByKey("data.legal.time");
        int legalTime = Integer.parseInt(config) * 1000;
        Date date = Date.from(startTime.toLocalDate().atStartOfDay(ZoneOffset.of("+8")).toInstant());
        //先处理上次未统计完整的数据
        TwinCalcAlarms calcSearch = new TwinCalcAlarms();
        calcSearch.setCalcStatus("1");
        List<TwinCalcAlarms> oldList = selectTwinCalcAlarmsList(calcSearch);
        Map<Long, List<TwinCalcAlarms>> oldMap = oldList.stream().collect(Collectors.groupingBy(TwinCalcAlarms::getDeviceId, LinkedHashMap::new, Collectors.toList()));
        //再处理本次数据，2次结合
        TwinRecordAlarms recordSearch = new TwinRecordAlarms();
        recordSearch.setDataDate(date);
        recordSearch.setHour(startTime.getHour());
        List<TwinRecordAlarms> newList = alarmsService.selectTwinRecordAlarmsList(recordSearch);
        Map<Long, List<TwinRecordAlarms>> newMap = newList.stream().collect(Collectors.groupingBy(TwinRecordAlarms::getDeviceId, LinkedHashMap::new, Collectors.toList()));

        List<TwinCalcAlarms> insertList = new ArrayList<>();
        List<TwinCalcAlarms> updateList = new ArrayList<>();
        List<TwinRecordAlarms> recordList = new ArrayList<>();
        //需要删除的数据
        List<Long> delIds = new ArrayList<>();
        deviceList.forEach(device -> {
            List<TwinRecordAlarms> list = new ArrayList<>();
            if (oldMap.get(device.getDeviceId()) != null) {
                oldMap.get(device.getDeviceId()).forEach(old -> {
                    TwinRecordAlarms alarms = new TwinRecordAlarms();
                    BeanUtils.copyProperties(old, alarms);
                    //这里存ID，方便后面需要删除，判断是否之前的记录，则直接判断remark是否为空即可
                    alarms.setRemark(old.getId() + "");
                    list.add(alarms);
                });
            }
            //再处理本次数据，2次结合
            if (newMap.get(device.getDeviceId()) != null) {
                list.addAll(newMap.get(device.getDeviceId()));
            }

            Map<Integer, List<TwinRecordAlarms>> groupList = list.stream().collect(Collectors.groupingBy(TwinRecordAlarms::getAlarmType));
            for (Map.Entry<Integer, List<TwinRecordAlarms>> entry : groupList.entrySet()) {
                List<TwinRecordAlarms> temp = entry.getValue();
                if (temp.size() == 1) {
                    //只有一条记录，则直接记录下来
                    recordList.add(temp.get(0));
                }
                TwinRecordAlarms lastAlarms = null;
                for (int i = 0; i < temp.size(); i++) {
                    TwinRecordAlarms currAlarms = temp.get(i);
                    if (lastAlarms == null) {
                        lastAlarms = currAlarms;
                    } else {
                        //1.第一轮，下一条开始时间-上一条结束时间小于 seconds 则两条记录合并
                        if (currAlarms.getStartTime().getTime() - lastAlarms.getEndTime().getTime() < legalTime) {
                            lastAlarms.setEndTime(currAlarms.getEndTime());
                            lastAlarms.setCalcStatus(currAlarms.getCalcStatus());
                        } else {
                            //2.判断持续时间，如果大于seconds则记录
                            if (lastAlarms.getEndTime().getTime() - lastAlarms.getStartTime().getTime() > legalTime) {
                                recordList.add(lastAlarms);
                            }
                            //如果无效数据是未统计完成的记录，则需要删除统计里面的数据
                            if (StringUtils.isNotEmpty(lastAlarms.getRemark())) {
                                delIds.add(Long.parseLong(lastAlarms.getRemark()));
                            }
                            lastAlarms = currAlarms;
                        }
                        if (i + 1 == temp.size()) {
                            //补一次最后一条记录,如果最后一条记录是统计未完成的，或者持续时间大于seconds的,则补录
                            if ("1".equals(lastAlarms.getCalcStatus()) || lastAlarms.getEndTime().getTime() - lastAlarms.getStartTime().getTime() > legalTime) {
                                recordList.add(lastAlarms);
                            }
                            //如果无效数据是未统计完成的记录，则需要删除统计里面的数据
                            if (StringUtils.isNotEmpty(lastAlarms.getRemark())) {
                                delIds.add(Long.parseLong(lastAlarms.getRemark()));
                            }
                        }
                    }
                }
            }
        });
        for (TwinRecordAlarms stop : recordList) {
            TwinCalcAlarms v = new TwinCalcAlarms();
            BeanUtils.copyProperties(stop, v);
            if (StringUtils.isNotEmpty(stop.getRemark())) {
                v.setCalcStatus("0");
                v.setRemark("");
                updateList.add(v);
            } else {
                v.setId(null);
                insertList.add(v);
            }
        }

        try (SqlSession sqlSession = factory.openSession(ExecutorType.BATCH, false)) {
            TwinCalcAlarmsMapper mapper = sqlSession.getMapper(TwinCalcAlarmsMapper.class);
            insertList.forEach(mapper::insertTwinCalcAlarms);
            updateList.forEach(mapper::updateTwinCalcAlarms);
            sqlSession.commit();
        }
        if (delIds.size() > 0) {
            deleteTwinCalcAlarmsByIds(delIds.toArray(new Long[]{}));
        }


        //统计完成删除记录数据，避免记录超长
        alarmsService.deleteTwinRecordAlarms(date, startTime.getHour());
    }

    /**
     * 按开始和结束时间查询数据
     *
     * @param sTime 开始时间
     * @param eTime 结束时间
     * @return 查询列表
     */
    @Override
    public List<TwinCalcAlarms> selectTwinCalcAlarmsListByDate(Date sTime, Date eTime) {
        TwinCalcAlarms alarms = new TwinCalcAlarms();
        Map<String, Object> params = new HashMap<>(16);
        params.put("sTime", sTime);
        params.put("eTime", eTime);
        alarms.setParams(params);
        return selectTwinCalcAlarmsList(alarms);
    }
}
