package com.jjt.order.controller;

import com.jjt.calc.domain.TwinCalcDay;
import com.jjt.calc.domain.TwinFormulaInfo;
import com.jjt.calc.service.ITwinCalcDayService;
import com.jjt.calc.service.ITwinFormulaInfoService;
import com.jjt.common.constant.CacheConstants;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.redis.RedisCache;
import com.jjt.common.utils.DateUtils;
import com.jjt.order.domain.TwinOrder;
import com.jjt.order.domain.TwinOrderDetail;
import com.jjt.order.domain.VmsStock;
import com.jjt.order.service.ITwinOrderService;
import com.jjt.order.utils.OrderExportUtil;
import com.jjt.order.vo.BomVO;
import com.jjt.order.vo.SkuVO;
import com.jjt.utils.MssqlUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ruoyi
 */
@Api("数据接口")
@RestController
@Slf4j
@RequestMapping()
public class OrderApiController extends BaseController {
    @Resource
    private RedisCache redisCache;
    @Resource
    private ITwinOrderService orderService;
    @Resource
    private ITwinFormulaInfoService formulaInfoService;
    @Resource
    private ITwinCalcDayService dayService;

    @ApiOperation("查看当前库存")
    @GetMapping("/api/stock")
    @CrossOrigin(origins = "*")
    @ResponseBody
    public Map<String, Object> mesStock() {
        return redisCache.getCacheObject(CacheConstants.VMS_STOCK);
    }

    @ApiOperation("导出白坯预测")
    @GetMapping("/api/export/order")
    @CrossOrigin(origins = "*")
    public void orderExport(String start, String end, HttpServletResponse response) {
        //查询订单
        List<TwinOrder> orderList = orderService.selectTwinOrderListByDate(start, end);
        //按订单ID分组
        Map<Long, TwinOrder> orderMap = orderList.stream().collect(Collectors.toMap(TwinOrder::getOderId, o -> o));
        List<TwinFormulaInfo> infos = formulaInfoService.selectTwinFormulaInfoList(new TwinFormulaInfo());
        LocalDateTime ldt = LocalDateTime.now();
        Date sd = DateUtils.toDate(LocalDate.now().minusDays(2));
        if (ldt.getHour() > 7) {
            sd = DateUtils.toDate(LocalDate.now().minusDays(1));
        }
        List<TwinCalcDay> dayList = dayService.selectTwinCalcDayListByTime(sd, sd);
        //按设备ID分组
        Map<Long, TwinCalcDay> dayMap = dayList.stream().collect(Collectors.toMap(TwinCalcDay::getDeviceId, o -> o));
        List<VmsStock> stocks = MssqlUtil.vmsStock();
        List<TwinOrderDetail> all = new ArrayList<>();
        orderList.forEach(to -> all.addAll(to.getTwinOrderDetailList().stream().filter(tod -> tod.getBomSpec() != null).collect(Collectors.toList())));
        //按规格分组
        Map<String, List<TwinOrderDetail>> allMap = all.stream().collect(Collectors.groupingBy(TwinOrderDetail::getBomSpec, LinkedHashMap::new, Collectors.toList()));
        List<BomVO> bomList = new ArrayList<>();
        List<SkuVO> skuList = new ArrayList<>();
        for (String spec : allMap.keySet()) {
            List<TwinOrderDetail> todList = allMap.get(spec);
            //匹配白坯布库存
            Float kc = 0f;
            VmsStock stock = matchStock(spec, stocks);
            if (stock != null) {
                kc = stock.getQty();
            }
            //匹配生产设备
            List<TwinFormulaInfo> matchFormula = matchFormula(spec, infos);
            BigDecimal cl = BigDecimal.ZERO;
            String deviceIds = "";
            for (TwinFormulaInfo match : matchFormula) {
                if (StringUtils.isNotEmpty(deviceIds)) {
                    deviceIds += ",";
                }
                deviceIds += match.getDeviceId();
                TwinCalcDay day = dayMap.get(match.getDeviceId());
                if (day != null) {
                    cl = cl.add(day.getWeight());
                }
            }

            cl = cl.multiply(BigDecimal.valueOf(1000));

            BomVO bom = new BomVO();
            bom.setDeviceIds(deviceIds);
            bom.setLast(cl.doubleValue());
            //统计订单数量
            long orders = todList.stream().map(TwinOrderDetail::getOderId).distinct().count();
            double totalWeight = todList.stream().mapToDouble(o -> o.getWeight().doubleValue()).sum();
            TwinOrderDetail tod = todList.get(0);
            bom.setBomCode(tod.getBomCode());
            bom.setBomName(tod.getBomName());
            bom.setBomSpec(spec);

            bom.setXql(totalWeight);
            bom.setKcl(kc);
            bom.setOrders(orders);
            double ycl = cl.multiply(BigDecimal.TEN).doubleValue();
            double cz = totalWeight - kc - ycl;
            bom.setYcl(ycl);
            bom.setCz(cz);
            bom.setCl(cz / totalWeight);

            SkuVO sku = new SkuVO();
            sku.setBomCode(tod.getBomCode());
            sku.setBomName(tod.getBomName());
            sku.setBomSpec(spec);
            if (stock != null) {
                sku.setSku(stock.getSku());
                sku.setDesc(stock.getDesc());
                sku.setNum(stock.getNum());
                sku.setQty(stock.getQty());
            } else {
                sku.setSku("");
                sku.setDesc("");
                sku.setNum(0);
                sku.setQty(0f);
            }
            sku.setXql(bom.getXql());
            List<TwinOrder> skuOrder = new ArrayList<>();
            //按订单分组
            Map<Long, List<TwinOrderDetail>> map = todList.stream().collect(Collectors.groupingBy(TwinOrderDetail::getOderId, LinkedHashMap::new, Collectors.toList()));
            for (Long oid : map.keySet()) {
                TwinOrder to = orderMap.get(oid);
                to.setTwinOrderDetailList(map.get(oid));
                skuOrder.add(to);
            }
            sku.setOrderList(skuOrder);
            bom.setOrderList(skuOrder);
            bomList.add(bom);
            skuList.add(sku);
        }

        try (InputStream inputStream = getClass().getClassLoader().getResourceAsStream("tpl/order.xlsx"); XSSFWorkbook wb = new XSSFWorkbook(inputStream); OutputStream outputStream = new BufferedOutputStream(response.getOutputStream())) {
            OrderExportUtil oe = new OrderExportUtil(wb);

            XSSFSheet sheet0 = wb.getSheetAt(0);
            Cell title = sheet0.getRow(0).getCell(0);
            title.setCellValue("白坯布靶向生产测算-10天（" + start + " 至 " + end + "）");

            oe.sheet0(sheet0, bomList);
            oe.sheet1(wb.getSheetAt(1), stocks);
            oe.sheet2(wb.getSheetAt(2), skuList);
            oe.sheet4(wb.getSheetAt(4), infos);
            response.reset();
            // 设置response的Header
            response.setCharacterEncoding("UTF-8");
            //Content-Disposition的作用：告知浏览器以何种方式显示响应返回的文件，用浏览器打开还是以附件的形式下载到本地保存
            //attachment表示以附件方式下载 inline表示在线打开 "Content-Disposition: inline; filename=文件名.mp3"
            // filename表示文件的默认名称，因为网络传输只支持URL编码的相关支付，因此需要将文件名URL编码后进行传输,前端收到后需要反编码才能获取到真正的名称
            response.addHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("白坯布靶向预测" + start + "至" + end + ".xlsx", "UTF-8"));
            response.setContentType("application/octet-stream");
            wb.write(outputStream);
            outputStream.flush();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
    }


    /**
     * 匹配库存
     *
     * @param spec   规格
     * @param stocks 库存
     * @return 结果
     */
    private VmsStock matchStock(String spec, List<VmsStock> stocks) {
        //按重量分组
        Map<Integer, List<VmsStock>> stockMap = stocks.stream().collect(Collectors.groupingBy(VmsStock::getWeight, LinkedHashMap::new, Collectors.toList()));
        if (spec != null) {
            String[] tmp = spec.split("\\*");
            if (tmp.length == 3) {
                float length = Float.parseFloat(tmp[0]);
                float width = Float.parseFloat(tmp[1]);
                int weight = (int) (Float.parseFloat(tmp[2]) * 1000);
                List<VmsStock> tempList = stockMap.get(weight);
                if (tempList != null) {
                    for (VmsStock stock : tempList) {
                        if (length > stock.getLength() || width > stock.getLength()) {
                            return stock;
                        }
                    }
                }
            }
        }

        return null;
    }

    /**
     * 匹配产线
     *
     * @param spec 规格
     * @param list 产线
     * @return 结果
     */
    private List<TwinFormulaInfo> matchFormula(String spec, List<TwinFormulaInfo> list) {
        List<TwinFormulaInfo> result = new ArrayList<>();
        //按重量分组
        Map<Float, List<TwinFormulaInfo>> formulaMap = list.stream().collect(Collectors.groupingBy(TwinFormulaInfo::getFd18, LinkedHashMap::new, Collectors.toList()));
        if (spec != null) {
            String[] tmp = spec.split("\\*");
            if (tmp.length == 3) {
                Float length = Float.parseFloat(tmp[0]);
                Float width = Float.parseFloat(tmp[1]);
                Float weight = Float.parseFloat(tmp[2]);
                List<TwinFormulaInfo> infos = formulaMap.get(weight);
                if (infos != null) {
                    for (TwinFormulaInfo info : infos) {
                        if (length.equals(info.getFd16()) && width.equals(info.getFd17()) ||
                                length.equals(info.getFd17()) && width.equals(info.getFd16())) {
                            result.add(info);
                        }
                    }
                }
            }
        }
        return result;
    }
}
