package com.jjt.ws.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.enums.BusinessType;
import com.jjt.ws.domain.TwinWorkshopCalc;
import com.jjt.ws.service.ITwinWorkshopCalcService;
import com.jjt.common.utils.poi.ExcelUtil;
import com.jjt.common.core.page.TableDataInfo;

/**
 * 能源统计Controller
 *
 * @author wukai
 * @date 2025-01-17
 */
@Api(tags="能源统计")
@RestController
@RequestMapping("/ws/calc")
public class TwinWorkshopCalcController extends BaseController{
    @Resource
    private ITwinWorkshopCalcService twinWorkshopCalcService;

    /**
     * 查询能源统计列表
     */
    @ApiOperation("查询能源统计列表")
    @PreAuthorize("@ss.hasPermi('ws:calc:list')")
    @GetMapping("/list")
    public TableDataInfo list(TwinWorkshopCalc twinWorkshopCalc)
    {
        startPage();
        List<TwinWorkshopCalc> list = twinWorkshopCalcService.selectTwinWorkshopCalcList(twinWorkshopCalc);
        return getDataTable(list);
    }

    /**
     * 导出能源统计列表
     */
    @ApiOperation("导出能源统计列表")
    @PreAuthorize("@ss.hasPermi('ws:calc:export')")
    @Log(title = "能源统计", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TwinWorkshopCalc twinWorkshopCalc)
    {
        List<TwinWorkshopCalc> list = twinWorkshopCalcService.selectTwinWorkshopCalcList(twinWorkshopCalc);
        ExcelUtil<TwinWorkshopCalc> util = new ExcelUtil<TwinWorkshopCalc>(TwinWorkshopCalc.class);
        util.exportExcel(response, list, "能源统计数据");
    }

    /**
     * 获取能源统计详细信息
     */
    @ApiOperation("获取能源统计详细信息")
    @PreAuthorize("@ss.hasPermi('ws:calc:query')")
    @GetMapping(value = "/{calcId}")
    public AjaxResult getInfo(@PathVariable("calcId") Long calcId)
    {
        return success(twinWorkshopCalcService.selectTwinWorkshopCalcByCalcId(calcId));
    }

    /**
     * 新增能源统计
     */
    @ApiOperation("新增能源统计")
    @PreAuthorize("@ss.hasPermi('ws:calc:add')")
    @Log(title = "能源统计", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TwinWorkshopCalc twinWorkshopCalc)
    {
        return toAjax(twinWorkshopCalcService.insertTwinWorkshopCalc(twinWorkshopCalc));
    }

    /**
     * 修改能源统计
     */
    @ApiOperation("修改能源统计")
    @PreAuthorize("@ss.hasPermi('ws:calc:edit')")
    @Log(title = "能源统计", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TwinWorkshopCalc twinWorkshopCalc)
    {
        return toAjax(twinWorkshopCalcService.updateTwinWorkshopCalc(twinWorkshopCalc));
    }

    /**
     * 删除能源统计
     */
    @ApiOperation("删除能源统计")
    @PreAuthorize("@ss.hasPermi('ws:calc:remove')")
    @Log(title = "能源统计", businessType = BusinessType.DELETE)
	@DeleteMapping("/{calcIds}")
    public AjaxResult remove(@PathVariable Long[] calcIds)
    {
        return toAjax(twinWorkshopCalcService.deleteTwinWorkshopCalcByCalcIds(calcIds));
    }
}
